@@ -1,12 +1,16 @@
 from bank_sync.Resources.resource import Resource
 from bank_sync.Resources.accounts import Accounts
 from bank_sync.Resources.payments import Payments
-from bank_sync.models import IPN
 from django.core.exceptions import ObjectDoesNotExist
+from bank_sync.Resources.helpers import IsDjangoAppClass
+
+_is_django_app = False
 try:
+    from bank_sync.models import IPN
     from django.conf import settings
+    _is_django_app = True
 except Exception as e:
-    pass
+    _is_django_app = False
 
 
 class Bank(Resource):
@@ -27,11 +31,9 @@ class Bank(Resource):
     # This IPN callback is set in the django settings module
     # When a user registeres for an IPN using this service the sync will save the user url and forward the service url to Nash core
     # Nash will then forward ipn data to this service whihc then forwards it to the saved/registered user url
-    try:
+    if _is_django_app:
         bank_sync_ipn_urls = getattr(
             settings, 'BANK_SYNC_IPN_URLS', bank_sync_ipn_urls)
-    except Exception as e:
-        pass
 
     def __init__(self, nash, bank_id=None):
         self._nash = nash
@@ -40,7 +42,11 @@ class Bank(Resource):
 
     def resource(self, resource_name):
         resource = self._resources[resource_name].set_bank_id(
-            super().get_bank_id()).set_headers(self._nash.get_headers())
+            super().get_bank_id()).set_headers(self._nash.get_headers())\
+            .set_oauth2_client_id(oauth2_client_id=super().get_oauth2_client_id())\
+            .set_client_secret(client_secret=super().get_client_secret())\
+            .set_auth_token_url(auth_token_url=super().get_auth_token_url())\
+            .set_auth_scope(auth_scope=super().get_auth_scope())
 
         return resource
 
@@ -95,6 +101,7 @@ class Bank(Resource):
     # We do this so that when an IPN is sent to us, we simply confirm if the if the IPN sent is coming from Nash
     # by comparing the security credentials and the account number sent at the endpoint responsible of standardizing
     # the response
+    @IsDjangoAppClass(_is_django_app)
     def ipn(self, operation=None, payload=None):
         # Get the user passed url
         user_url = payload.get('url', '')
@@ -109,7 +116,7 @@ class Bank(Resource):
 
         try:
 
-            if operation in [super().IPN_REGISTER]:
+            if operation in [super().IPN_REGISTER] and 'error' not in super().response().keys():
                 # get the response and change the url saved, which it the service url, with the user url
                 rsp = super().response()
                 rsp['url'] = user_url
@@ -122,8 +129,8 @@ class Bank(Resource):
                     signature = self.generate_signature(secret=super().response().get(
                         'client_secret'), message=f"{super().response().get('client_id')}:{super().response().get('client_secret')}")
 
-                    IPN.objects.create(bank_id=payload.get('bank_id', ''), client_id=super().response().get(
-                        'client_id', ''), signature=signature, country_code=payload.get('country_code', ''), account_number=payload.get('account_number', ''), url=user_url)
+                    IPN.objects.create(bank_id=payload.get('bank_id', ''), tenant_id=payload.get('tenant_id', None), business_id=payload.get('business_id', None), client_id=super().response().get(
+                        'client_id', ''), signature=signature, country_code=payload.get('country_code', ''), currency_code=payload.get('currency_code', ''), account_number=payload.get('account_number', ''), url=user_url)
 
             # If the operation performed is to update an IPN
             elif operation == super().IPN_GET:
@@ -137,6 +144,8 @@ class Bank(Resource):
                     #     data = data.first()
 
                     rsp = super().response()
+                    rsp['bank_id'] = data.bank_id
+                    rsp['tenant_id'] = data.tenant_id
                     rsp['url'] = data.url
                     super().set_response(response=rsp)
 
@@ -161,7 +170,7 @@ class Bank(Resource):
 
                     data = IPN.objects.get(
                         client_id=payload.get('client_id', ''))
-                        
+
                     if 'bank_id' in payload.keys():
                         data.bank_id = payload.get('bank_id')
                     if 'account_number' in payload.keys():
@@ -169,12 +178,18 @@ class Bank(Resource):
                             'account_number')
                     if 'country_code' in payload.keys():
                         data.country_code = payload.get('country_code')
+                    if 'currency_code' in payload.keys():
+                        data.currency_code = payload.get('currency_code')
                     if 'url' in payload.keys():
                         data.url = user_url
                     if 'status' in payload.keys():
                         data.status = payload.get('status')
                     if 'ipn_type' in payload.keys():
                         data.ipn_type = payload.get('ipn_type')
+                    if 'tenant_id' in payload.keys():
+                        data.tenant_id = payload.get('tenant_id')
+                    if 'business_id' in payload.keys():
+                        data.business_id = payload.get('business_id')
 
                     data.save()
 
@@ -185,11 +200,11 @@ class Bank(Resource):
 
                     data = IPN.objects.get(client_id=payload.get(
                         'client_id', ''))
-                        
+
                     data.delete()
 
         except ObjectDoesNotExist:
-            print("Either the blog or entry doesn't exist.")
+            print("Either the I.P.N. or entry doesn't exist.")
 
         return exec_global
 
