@@ -2,7 +2,8 @@ class Operations(object):
 
     # These are the banking integrations
     # Each with a unique ID
-    GLOBAL = 0
+    AUTHENTICATE = -1
+    GLOBAL = AUTHENTICATE + 1
     COOP = GLOBAL + 1
     EQUITY = COOP + 1
     NCBA = EQUITY + 1
@@ -58,10 +59,23 @@ class Operations(object):
     JOBS_ACCOUNT_BALANCES = IPN_GENERATE_CLIENT_SECRET + 1
     JOBS_ACCOUNT_STATEMENT = JOBS_ACCOUNT_BALANCES + 1
 
+    AUTH_GET_TOKEN = JOBS_ACCOUNT_STATEMENT + 1
+    AUTH_AUTHORIZE = AUTH_GET_TOKEN + 1
+
     # All end points are placed in the BANKS_CONF
     # This is to allow easy standardization of the base url, endpoint and operations
     # When calling the APIs, in this case this is done in the resource class read function
     BANKS_CONF = {
+        AUTHENTICATE: {
+            "url": "https://authserver.purplecliff-03d4fbdd.westeurope.azurecontainerapps.io",
+            "read": {
+                "operations": {
+                    AUTH_GET_TOKEN: ("connect/token", "POST"),
+                    AUTH_AUTHORIZE: ("connect/authorize", "POST"),
+                }
+            }
+
+        },
         GLOBAL: {
             "url": "https://dev.bankingservices.nashglobal.co",
             "read": {
