@@ -3,19 +3,28 @@ from bank_sync.Resources.operations import Operations
 from bank_sync.APIs.api_format import API
 import json
 from bank_sync.APIs.utils.generate_code import get_code
-from bank_sync.models import Callbacks
-from bank_sync.models import IPNData
 import hmac
 import hashlib
 import base64
 import time
 import threading
-from bank_sync.models import IPN
 from django.core.exceptions import ObjectDoesNotExist
+from bank_sync.Resources.helpers import IsDjangoAppClass
+
+_is_django_app = False
+try:
+    from bank_sync.models import Callbacks
+    from bank_sync.models import IPNData
+    from bank_sync.models import IPN
+    _is_django_app = True
+except Exception as e:
+    _is_django_app = False
 
 # This class will be inherited by sub entities e.g. Accounts Class and Payments Class
 # It has common methods that these entities will override and implement
 # As wll as methods that can be used by any of the Entities
+
+
 class Resource(API, Operations):
 
     _bank_id = 0
@@ -41,6 +50,11 @@ class Resource(API, Operations):
 
     _ipn_object = None
 
+    # use this to add additonal payload to be sent to a callback
+    callback_payload = {}
+    # use this to add additonal headers to be sent to a callback
+    callback_headers = {}
+
     @property
     def READ(self):
         return 0
@@ -135,10 +149,11 @@ class Resource(API, Operations):
             endpoint = bank.get(self.get_action(), {}).get('endpoint', '')
 
             operation, method = bank.get(self.get_action(), {}).get(
-                'operations', {}).get(self.get_operation(), (-1,-1))
-            
-            if operation==-1 and method==-1:
-                self._response = {'error': f'Bank ID {self.get_bank_id()} Does not have Operation {self.get_operation()} in {self.get_action()} Entity'}
+                'operations', {}).get(self.get_operation(), (-1, -1))
+
+            if operation == -1 and method == -1:
+                self._response = {
+                    'error': f'Bank ID {self.get_bank_id()} Does not have Operation {self.get_operation()} in {self.get_action()} Entity'}
             else:
                 # Check if any query params where passed, while creating the end point
                 if bool(params):
@@ -146,7 +161,8 @@ class Resource(API, Operations):
                 else:
                     endpoint = f'{endpoint}/{operation}'
 
-                super().set_full_url(full_url=f"{bank.get('url','')}/{endpoint}")
+                super().set_full_url(
+                    full_url=f"{bank.get('url','')}/{endpoint}")
 
                 # Call the method exec to make the request to A.P.I. endpoint and return the data that will be returned in this method
                 self._response = self._exec(payload, method)
@@ -179,7 +195,7 @@ class Resource(API, Operations):
     # This method is used to simulate callbacks for banks or operations that return data synchronously
     # This method is also used by banks that do not return a transaction reference in the synchronous data, otherwise this
     # would have been handled by the core integrations
-
+    @IsDjangoAppClass(_is_django_app)
     def simulate_callback(self, response={}):
         if self.get_bank_id() in [super().EQUITY]:
             if self.get_operation() not in [super().MOBILE_WALLET]:
@@ -191,7 +207,7 @@ class Resource(API, Operations):
                 # Equity bank retunrs data synchronously, we will append to it unique identifiers that are not returned
                 # e.g. our unique bank id and type of operation/transaction performed
                 if self.get_bank_id() == super().EQUITY:
-                    response["code"] = int(response.get("code"))
+                    response["code"] = f'{int(response.get("code"))}'
                     response["completed"] = False
                     response["account_reference"] = request.get(
                         "transfer", {}).get("reference", "")
@@ -201,18 +217,30 @@ class Resource(API, Operations):
                         "destinations", [])[0].get("account_number", "")
                     response["amount"] = f'{request.get("destinations", [])[0].get("amount", -1)}'
 
-                    if response.get("code") == 0:
+                    if int(response.get("code")) == 0:
                         response["completed"] = True
 
                     callback_url = request.get(
                         "transfer", {}).get("callback_url", "")
 
+                    # we are setting additional data that needs to be sent back to the callback url
+                    # they are optional data
+                    if request.get("businessid", None) is not None:
+                        self.callback_payload = {
+                            'businessid': request.get("businessid", None)
+                        }
+                    if request.get("__tenant", None) is not None:
+                        self.callback_headers = {
+                            '__tenant': request.get("__tenant", None)
+                        }
+
                     # simultate a callback by sending data later via a thread
                     # this will only work if a callback was sent/placed in the standardized payload
                     threading.Thread(target=self._send_to_callback,
-                                     args=[response, callback_url]).start()
+                                     args=[response, callback_url, self.callback_payload, self.callback_headers]).start()
 
     # This method is used to standardize callbacks received
+    @IsDjangoAppClass(_is_django_app)
     def standardize_callback(self, callback_data={}, forward=True):
 
         response = {
@@ -236,19 +264,20 @@ class Resource(API, Operations):
 
             if callback_data.get("bank_id", -1) == super().COOP:
 
+                callback = saved_callback
+
                 if callback_data.get("type", -1) in [super().IFT, super().MOBILE_WALLET, super().PESALINK_TO_BANK, super().PESALINK_TO_MOBILE]:
 
                     # if len(saved_callback) == 0:
-                    if len(callback_data.get("Destinations", [])):
-                        callback = Callbacks.objects.get(reference=callback_data.get(
-                            "Destinations", [])[0].get("ReferenceNumber", ""))
+                    # if len(callback_data.get("Destinations", [])):
+                    #     callback = Callbacks.objects.get(reference=callback_data.get(
+                    #         "Destinations", [])[0].get("ReferenceNumber", ""))
 
                     # if len(saved_callback):
                     #     callback = saved_callback.first()
-                    callback = saved_callback
+                    # callback = saved_callback
 
-                    response["code"] = int(callback_data.get(
-                        "MessageCode", callback_data.get("messageCode", -1)))
+                    response["code"] = f'{int(callback_data.get("MessageCode", callback_data.get("messageCode", -1)))}'
                     response["account_reference"] = callback_data.get(
                         "reference", "")
                     response["date"] = callback_data.get(
@@ -279,6 +308,9 @@ class Resource(API, Operations):
                             if callback.type_code == super().MOBILE_WALLET:
                                 response["account_destination"] = callback_data.get(
                                     "Destinations", [])[0].get("MobileNumber", "")
+                            elif callback.type_code == super().PESALINK_TO_MOBILE:
+                                response["account_destination"] = callback_data.get(
+                                    "Destinations", [])[0].get("PhoneNumber", "")
 
                     elif 'destination' in callback_data.keys():
                         response["account_destination"] = callback_data.get(
@@ -303,19 +335,21 @@ class Resource(API, Operations):
                     elif 'destination' in callback_data.keys():
                         response["amount"] = f'{callback_data.get("destination", {}).get("amount", "")}'
 
-                    if response.get("code") == 0:
+                    if int(response.get("code")) == 0:
                         response["completed"] = True
 
             elif callback_data.get("bank_id", -1) == super().EQUITY:
 
+                callback = saved_callback
+
                 if callback_data.get("type", -1) == super().MOBILE_WALLET:
 
                     # if len(saved_callback):
                     #     callback = saved_callback.first()
-                    callback = saved_callback
+                    # callback = saved_callback
                     response.update({
                         "message": f'{callback_data.get("message", "")}: {callback_data.get("data", {}).get("ResponseDescription","")}',
-                        "code": round(float(callback_data.get("code", -1)), 2),
+                        "code": f'{round(float(callback_data.get("code", -1)), 2)}',
                         "transaction_id": callback_data.get("data", {}).get("ThirdPartyTranID", ""),
                         "date": callback.request.get("transfer", {}).get('date', ""),
                         "account_reference": callback_data.get("transactionReference", ""),
@@ -324,13 +358,13 @@ class Resource(API, Operations):
                         "amount": round(float(callback.request.get("source", {}).get('amount', -1)), 2)
                     })
 
-                    if response.get("code") == 0:
+                    if int(float(response.get("code"))) == 0:
                         response["completed"] = True
 
                 elif callback_data.get("type", -1) == super().PDF_TO_JSON:
                     # if len(saved_callback):
                     #     callback = saved_callback.first()
-                    callback = saved_callback
+                    # callback = saved_callback
                     response = self.standardize_pdf_to_json(
                         response_data=callback_data)
 
@@ -338,15 +372,27 @@ class Resource(API, Operations):
             # If it was supplied ans saved, else return the standardized response back to the
             if forward:
                 if callback is not None:
+                    # we are setting additional data that needs to be sent back to the callback url
+                    # they are optional data
+                    if callback.business_id is not None:
+                        self.callback_payload = {
+                            'businessid': str(callback.business_id)
+                        }
+                    if callback.tenant_id is not None:
+                        self.callback_headers = {
+                            '__tenant': str(callback.tenant_id)
+                        }
+                    
                     self._send_to_callback(
-                        response, callback_url=callback.callback, sleep=False)
+                        response, callback_url=callback.callback, callback_headers=self.callback_headers, callback_payload=self.callback_payload, sleep=False)
 
         except ObjectDoesNotExist:
-            print("Either the blog or entry doesn't exist.")
+            print("Either the callback or entry doesn't exist.")
 
         return response
 
     # This method is used to standardize IPN responses received
+    @IsDjangoAppClass(_is_django_app)
     def standardize_ipn(self, ipn_data={}, forward=True):
 
         ipn = None
@@ -378,7 +424,7 @@ class Resource(API, Operations):
 
                     response.update({
                         "message": f'{ipn_data.get("transaction", {}).get("status","")} {ipn_data.get("transaction", {}).get("remarks", "")}',
-                        "code": ipn_data.get("code", -1),
+                        "code": f'{ipn_data.get("code", "-1")}',
                         "transaction_id": ipn_data.get("transaction", {}).get("reference", ""),
                         "date": ipn_data.get("transaction", {}).get('date', ""),
                         "account_reference": f'{ipn_data.get("customer", {}).get("reference", "")}',
@@ -393,18 +439,28 @@ class Resource(API, Operations):
                             "transaction", {}).get("remarks", "")
                         response["account_destination"] = f"{ipn_data.get('bank',{}).get('account','')}",
 
-                    if response.get("code") == 0:
+                    if int(response.get("code")) == 0:
                         response["completed"] = True
 
                 # This is a variable used to forward the standardized ipn data to another callback url
                 # If it was supplied and saved, else return the standardized response back to the
                 if forward:
                     if ipn is not None:
+                        # we are setting additional data that needs to be sent back to the callback url
+                        # they are optional data
+                        if ipn.business_id is not None:
+                            self.callback_payload = {
+                                'businessid': str(ipn.business_id)
+                            }
+                        if ipn.tenant_id is not None:
+                            self.callback_headers = {
+                                '__tenant': str(ipn.tenant_id)
+                            }
                         self._send_to_callback(
-                            response, callback_url=ipn.url, sleep=False)
+                            response, callback_url=ipn.url, callback_headers=self.callback_headers, callback_payload=self.callback_payload, sleep=False)
 
         except ObjectDoesNotExist:
-            print("Either the blog or entry doesn't exist.")
+            print("Either the IPN or entry doesn't exist.")
 
         return response
 
@@ -776,10 +832,13 @@ class Resource(API, Operations):
     # Use this method to set an operation/transaction to save/register callbacks
     # When an operation is called with this method, it saves the callback url sent and the reference
     # This method regesiters callbacks for banks that return data as a callback with the reference sent
+    @IsDjangoAppClass(_is_django_app)
     def register_callback(self):
         save_callback = True
         reference = None
         callback = None
+
+        # check if the tenant id and business id are passed
         if self.get_bank_id() in [super().COOP, super().EQUITY]:
             if super().EQUITY == self.get_bank_id() and self.get_operation() not in [super().MOBILE_WALLET, super().PDF_TO_JSON]:
                 save_callback = False
@@ -787,6 +846,7 @@ class Resource(API, Operations):
         if save_callback:
             # Get the request that was sent
             request = self.request()
+
             # save the reference, callback, bank id and operation
             if super().EQUITY == self.get_bank_id() and self.get_operation() == super().PDF_TO_JSON:
                 reference = request.get("reference", "")
@@ -796,11 +856,12 @@ class Resource(API, Operations):
                 callback = request.get("transfer", {}).get("callback_url", "")
 
             if reference != '' and callback != '':
-                Callbacks.objects.create(reference=reference,
+                Callbacks.objects.create(reference=reference, tenant_id=request.get("__tenant", None), business_id=request.get("businessid", None),
                                          callback=callback,
                                          request=request, bank_id=self.get_bank_id(), type_code=self.get_operation())
 
     # This method is used to save callback responses received
+    @IsDjangoAppClass(_is_django_app)
     def save_callback_data(self, saved_callback, callback_data):
         callback = saved_callback
         callback.response = callback_data
@@ -808,6 +869,7 @@ class Resource(API, Operations):
         callback.save()
 
     # This method is used to save IPN responses received
+    @IsDjangoAppClass(_is_django_app)
     def save_ipn_data(self, saved_ipn, ipn_data):
         saved_ipn = saved_ipn
         IPNData.objects.create(ipn=saved_ipn, response=ipn_data)
@@ -819,13 +881,26 @@ class Resource(API, Operations):
         return base64.b64encode(digest).decode()
 
     # Use this method to send data to a saved callback URL
-    def _send_to_callback(self, response, callback_url='', sleep=True, sleep_time=1):
+    def _send_to_callback(self, response, callback_url='', callback_payload={}, callback_headers={}, sleep=True, sleep_time=1):
+        # if sleep has been set to true, sleep for the specified sleep time
         if sleep:
             time.sleep(1*sleep_time)
+        # check if there is a callback url
         if callback_url != '':
-            self.set_headers(headers={
-                'Content-Type': 'application/json'})
+            # check if there is additional data that you want to append to the response to be sent
+            if bool(callback_payload):
+                response.update(callback_payload)
+
+            # set the response you want to be sent
             payload = json.dumps(response)
+
+            # set the headers you want to be sent
+            headers = {'Content-Type': 'application/json'}
+            # check if there is additional headers that you want to append to the headers to be sent
+            if bool(callback_headers):
+                headers.update(callback_headers)
+
+            self.set_headers(headers=headers)
             self.set_full_url(callback_url)
             self.api_request(payload=payload,
                              method='POST', verify=True)
